@@ -27,6 +27,10 @@ class scpQCA:
             log_odds=[i*(3/(full_membership-cross_over)) if i>0 else i*(-3/(full_nonmembership-cross_over)) for i in deviation]
             return [round(np.exp(i)/(1+np.exp(i)),3) for i in log_odds]
         self.raw_data=copy.deepcopy(self.data)
+        if self.caseid !=None and self.caseid in self.feature_list:
+            self.feature_list.remove(self.caseid)
+        # if self.decision_name in self.feature_list:
+        #     self.feature_list.remove(self.decision_name)
         for factor in self.feature_list:
             self.data[factor]=np.array(Ragin_direct(list(self.data[factor]), full_membership, cross_over, full_nonmembership))
 
@@ -45,10 +49,11 @@ class scpQCA:
                         break
             return cali
         self.raw_data=copy.deepcopy(self.data)
-        character_list=copy.deepcopy(self.feature_list)
-        if self.caseid !=None and self.caseid in character_list:
-            character_list.remove(self.caseid)
-        for factor in character_list:
+        if self.caseid !=None and self.caseid in self.feature_list:
+            self.feature_list.remove(self.caseid)
+        # if self.decision_name in self.feature_list:
+        #     self.feature_list.remove(self.decision_name)
+        for factor in self.feature_list:
             # print(factor)
             self.data[factor]=np.array(Ragin_indirect(list(self.data[factor]),class_num, full_membership, full_nonmembership))
     
@@ -179,6 +184,7 @@ class scpQCA:
                     continue
                 else:
                     self.omit.append(str(factor)+'=='+str(value))
+        print("The filter items are:")
         print(self.omit)
 
     def candidate_rules(self,decision_label,consistency,cutoff,rule_length=5):
@@ -194,7 +200,7 @@ class scpQCA:
                 candidate.append(i)  
         print("Running...please wait. There are {} candidate combinations.".format(len(candidate)))
         rules=[]
-        for i in range(len(candidate)):  
+        for i in range(len(candidate)):
             length=len(candidate[i])
             Cartesian=[]                  
             for j in range(length):
@@ -205,17 +211,20 @@ class scpQCA:
                 for j in range(length):
                     Q+=str(candidate[i][j])+'=='+str(values[r][j])+' & '
                 Q=Q[:-3]
+                # print(Q)
                 flag=False
                 for n in self.necessity+self.omit:
                     if n in Q:
+                        # print(n)
                         flag=True
                 result=self.data.query(Q)
                 p=result[self.decision_name].value_counts(normalize = True, dropna = False)
-                if len(p)==0 or flag:
+                if len(result)==0 or flag:
+                    # print(len(result),flag)
                     pass
                 elif p.idxmax()==self.decision_label and p[p.idxmax()]>=consistency and p[p.idxmax()]*len(result)>=cutoff: 
-                    row=[Q,p[p.idxmax()]*len(result)/issues,p[p.idxmax()]] # coverage, consistency
-                    rules.append(row)
+                        row=[Q,p[p.idxmax()]*len(result)/issues,p[p.idxmax()]] # coverage, consistency
+                        rules.append(row)
         print("There are {} candidate rules in total.".format(len(rules)))
         return rules
 
@@ -705,6 +714,31 @@ class scpQCA:
         return
 
 # if __name__=="__main__":
+    # data=pd.read_csv("/Users/fumanqing/Documents/work/CCDA/2022.7论坛/misc/new_calibration.csv")
+    # obj1=scpQCA(data=data, feature_list=list(data.columns), decision_name='reginnovation', caseid='case')
+    # obj1.indirect_calibration(2)
+    # print(obj1.data)
+    # obj1.search_necessity(decision_label=1,consistency_threshold=0.9)
+    # obj1.filter()
+    # rules=obj1.candidate_rules(decision_label=1,consistency=0.8,cutoff=5,rule_length=5)
+    # obj1.scp_truth_table(rules=rules,decision_label=1)
+    # configuration,issue_set=obj1.greedy(rules,decision_label=1,unique_cover=5)
+    # print(configuration)
+    # print(issue_set)
+    # obj1.cov_n_con(configuration=configuration,issue_sets=issue_set)
+
+    # data=pd.read_csv("/Users/fumanqing/Documents/work/CCDA/2022.7论坛/misc/format.csv")
+    # obj1=scpQCA(data=data, feature_list=list(data.columns), decision_name='LC', caseid='cases')
+    # obj1.indirect_calibration(2)
+    # # print(obj1.data)
+    # obj1.search_necessity(decision_label=1,consistency_threshold=0.9)
+    # obj1.filter()
+    # rules=obj1.candidate_rules(decision_label=1,consistency=0.8,cutoff=2,rule_length=5)
+    # obj1.scp_truth_table(rules=rules,decision_label=1)
+    # configuration,issue_set=obj1.greedy(rules,decision_label=1,unique_cover=1)
+    # print(configuration)
+    # print(issue_set)
+    # obj1.cov_n_con(configuration=configuration,issue_sets=issue_set)
 #     data=[[random.randint(0,100) for _ in range(6)] for _ in range(30)]
 #     data=pd.DataFrame(data)
 #     data.columns=['A','B','C','D','F','cases']
